# #
# $Date: 1996/04/11 11:56:36 $  $Author: frankp $   $Revision: 1.22 $ #
# #
# frankp, 20.09.1994 #

#++
  nulspace.mu

	linalg::nullSpace  --  compute the kernel of a matrix

	nullSpace(A)

	A: matrix

	linalg::nullSpace(A) computes a basis for the kernel of A.
	A must belong to the category Cat::Field.
++#

linalg::nullSpace := proc(A)
    name linalg::nullSpace;
    local R, c, r, _ns, ns, rs, k, v, i, m,
	  Rzero, Rone, Rnegate, newThis;
begin
    if testargs() then
	if args(0) <> 1 then
	    error("wrong no of args")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("expecting a matrix")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::Field ) then
	    error("expecting a matrix over a Cat::Field")
	end_if
    end_if;

    userinfo(1,"compute kernel of a matrix");

    if A::hasProp( Cat::SquareMatrixCat ) then
        newThis:= (A::getSuperDomain(A))::newThis
    else
	newThis:= A::newThis
    end_if;

    k:= A::dimen(A);
    r:= k[1]; c:= k[2];

    userinfo(2,"compute Gauss-Jordan form");
    A:= linalg::gaussJordan( A,hold(All) );

    ns:= A[4]; A:= A[1];

    if nops( ns ) = 0 then
	return( [] )
    end_if;

    userinfo(2,"solve homogeneous system");
    R:= A::coeffRing;
   
    Rnegate:= R::negate;
    Rone:= R::one;
    Rzero:= R::zero;

    _ns:= {};
    rs:= [];
    for k in ns do
	v:= newThis( c,1 );
	i:= 0;
	for m from 1 to c do
 	    # has( _ns, m ) ? #
	    if _ns intersect {m} = {m} then i:= i + 1
	    elif m-i <= r then v[m,1]:= Rnegate( A[m-i,k] )
	    else v[m,1]:= Rzero
	    end_if
        end_for;
	v[k,1]:= Rone;
	rs:= append( rs,v );

	_ns:= _ns union {k}
    end_for;

    rs
end_proc:

# end of file #
