# product(f(k),k) returns g(k) such that g(k+1)/g(k)=f(k)
Examples:
product(k^2,k=1..4);

                                    576
product(k^2,k);
                                         2
                                 gamma(k) 
product(a[k],k=1..4);
                            a[1] a[2] a[3] a[4]
product(n+k,k=1..4);
                      (n + 1) (n + 2) (n + 3) (n + 4)
product(n+k,k);
                               gamma(k + n)
product(n+k,k=1..m);
                             gamma(m + n + 1)
                             ----------------
                               gamma(n + 1)  
product(2*h(k+1)/h(k),k);

                                        k
                                  h(k) 2 
#
product := proc(f,k)
begin
   if f::product<>FAIL then f::product(args())
   elif type(k)=DOM_IDENT then stdlib::prod_indef(f,k)
   elif testtype(k,hold(_equal)(DOM_IDENT,"_range")) then
      stdlib::prod_def(f,op(k,1),op(op(k,2)))
   else error("invalid argument")
   end_if
end_proc:

# indefinite product, k is an identifier #
stdlib::prod_indef := proc(f,k) local a,b,c;
begin
   if not has(f,k) then return(f^k)
   elif type(f)="_mult" then
      a:=numer(f); b:=denom(f); 
      if not has((c:=a/subs(b,k=k+1)),k) then return(b*c^k)
      else return(map(f,stdlib::prod_indef,k))
      end_if
   elif type(f)="_power" and not has(op(f,2),k) then
      return(stdlib::prod_indef(op(f,1),k)^op(f,2))
   elif testtype(f,Type::PolyExpr(k)) then
      if degree(f,[k])=1 then
         a:=coeff(f,[k],1); b:=coeff(f,[k],0); return(a^k*gamma(k+b/a))
      elif contains({"_mult","_power"},type((a:=Factor(f)))) then
            return(stdlib::prod_indef(a,k))
      end_if
   end_if;
   hold(product)(f,k)
end_proc:

stdlib::prod_def := proc(f,k,a,b) local p,i,g;
begin
   if type(b-a)=DOM_INT then # finite product #
     p:=1;
      if b-a>=0 then for i from 0 to b-a do p:=p*subs(f,k=a+i) end_for
      else for i from 1 to a-b-1 do p:=p/subs(f,k=b+i) end_for
      end_if; return(p)
   else
      g:=stdlib::prod_indef(f,k);
      if not hastype(g,"product") then
         if subs(f,k=a)=0 then return(0) end_if;
         return(eval(subs(g,k=b+1)/subs(g,k=a)))
      end_if
   end_if;
   hold(product)(f,k=a..b)
end_proc:

product:=funcattr(product,"type","product"):
product:=funcattr(product,"print","product"):
