/* $Date: 1996/10/08 18:02:19 $  $Author: kg $  $Revision: 1.9 $ */

/******************************************************************************/
/* DATEI:       MDM_base.h                                                    */
/* AUTOR:       Andreas Sorgatz                                               */
/* ERSTELLT:    06/01/93                                                      */
/* GEAENDERT:   18/09/96                                                      */
/* INHALT:      Definitionen zur system-unab-haengigen Modul-Verwaltung.      */
/*                                                                            */
/* Diese Datei enthaelt Basis-Definitionen der Modul-Verwaltung und wird so-  */
/* wohl vom MuPAD-Kern, als auch von jedem MuPAD-Modul benoetigt.             */
/*                                                                            */
/******************************************************************************/

#ifndef __MDM_base__
#define __MDM_base__


#ifdef DOS_FILENAMES
#  include "MMMstora.h"
#else
#  include "MMMstorage.h"
#endif


/*** Der Modulgenerator hat eine 3-stellige Versionsnummer, die er in jedes ***/
/*** Modul eintraegt.  Stimmen die beiden hoestwertigen Ziffern dieser Num- ***/
/*** mer mit der im folgenden definierten Kernversion ueberein,  so ist die ***/
/*** Kompatibilitaet gewaehrleistet. Andernfalls sollte das Modul NICHT ge- ***/
/*** laden werden.                                                          ***/

#define MDMC_RELEASE            1300UL          /*** ==> MMG: Release 1.30x ***/
#define MDM_BAD_RELEASE(r)      ((r/10UL) != (MDMC_RELEASE/10UL))


/*** MDM-Infostring (fuer den Fall, dass vom Modul keiner gegeben wurde *******/

#define MDMC_MODINFO     "Modul: unknown\nInfo : unknown";


/*** MDM-Fehlerkodes **********************************************************/

#define MDMC_OK          0
 
#define MDMC_CLOSE       1
#define MDMC_EMPTY       2
#define MDMC_EXIST       3
#define MDMC_FULL        4
#define MDMC_INIT        5
#define MDMC_LOAD        6
#define MDMC_MEM         7
#define MDMC_READ        8
#define MDMC_TYPE        9
#define MDMC_UNDEF      10
#define MDMC_UNKNOWN    11
#define MDMC_USED       12
#define MDMC_GEN	13
#define MDMC_REL	14
#define MDMC_NAME	15


/*** Modul-Flags **************************************************************/

#define MDMC_FLAG_ZERO	 0
#define MDMC_NO_UNLOAD	 1                     /*** Modul niemals ausladen ***/
#define MDMC_DO_UNLOAD	 2            /*** Modul schnellstmoeglich ausladen ***/
#define MDMC_PSEUDO	 4            /*** Pseudo-Modul (statisch gelinkt)  ***/


/*** Laengen fuer Listen und Puffer *******************************************/

#define MDMC_LIST_MAX   256            /*** Standard fuer max. Modul-Anzahl ***/

#define MDM_BUF_LEN    2048            /*** Laenge fuer temporaere Puffer *****/


/*** Typen der Modulverwaltung ************************************************/

typedef S_Pointer (*MDM_jump_t)           /* Zeiger auf Standard-Kern-Routine */
                        ( S_Pointer           s,         
                          long                prev_func, 
                          long                eval_type, 
			  S_Pointer	      exec        
			);

typedef struct MDM_fun_s {              /* Funktions-Verwaltungs-Struktur     */
        char            *name;          /* Der (MuPAD-)Name der Funktion      */
        MDM_jump_t       jump;          /* ggf. Einsprungadresse der Funktion */
} MDM_fun_t;

typedef long (*MDM_pinit_t)      /* Zeiger auf (Pseudo-)Modul-Initialisierung */
			( unsigned long   release,
			  char          **info,
			  unsigned long  *flags,
			  long           *funcs, 
			  MDM_fun_t     **func_list,
			  MDM_jump_t    (*kernel)( CONST unsigned long index )
			);

typedef struct MDM_mod_s {              /* Modul-Verwaltungs-Struktur         */
        unsigned long    mmg_release;   /* Kompatibilitaetskode: 3st. Release */
        char            *name;          /* Der (MuPAD-)Name des Moduls        */
        char            *info;          /* Der Infostring zum Modul           */
        void            *handle;        /* System-ab-haengiger Modul-Handle   */
        MDM_fun_t       *func;          /* Vektor der Funktions-Handle        */
        long             funcs;         /* Anz. der Modul-Funktionen          */
        unsigned long	 flags;         /* Flags des Moduls                   */
        long             active;        /* Anz. aktiver Modul-Funktionen      */
        unsigned long    access;        /* Zeit des letzten Zugriffs          */
} MDM_mod_t ;

typedef struct MDM_list_s {             /* Modul-Verwaltungs-Listen-Struktur  */
        MDM_mod_t      **mod;           /* Vektor der Moduleintraege          */
        long             len;           /* Anz. der geladenen Module          */
        long             max;           /* Maximale Anz. geladener Module     */
} MDM_list_t ;

typedef struct MDM_pmod_s {             /* Pseudomodul-Verwaltungs-Struktur   */
	char            *name;          /* (MuPAD-)Name des Pseudomoduls      */
	MDM_pinit_t	*init;          /* Modul Initialisierungfunktion      */
} MDM_pmod_t ;


/*** Prototypen oeffentlicher Verwaltungs-Variablen (Kern) ********************/

extern MDM_list_t	*MDMV_list;

extern CONST long	 MDMV_support;
extern CONST long	 MDMV_unload_support;

extern unsigned long	 MDMV_age_max;
extern unsigned long	 MDMV_aging;


/*** Prototypen oeffentlicher Verwaltungs-Funktionen (Kern) *******************/

extern long		 MDM_init       ( CONST long list_max );
extern void		 MDM_exit       ( VOID );
extern long		 MDM_reset      ( VOID );

extern long		 MDM_kick_out   ( VOID );
extern void		 MDM_aging	( VOID );
extern void		 MDM_aging_intr	( VOID );

extern long		 MDM_load	( CONST char	 *name, 
					  MDM_mod_t	**modul );
extern long		 MDM_unload	( CONST char	 *name,
					  MDM_mod_t	 *modul,
					  long		  force );
extern long		 MDM_call	( CONST char	 *mod_name,
					  CONST char	 *fun_name,
					  S_Pointer	 *result,
					  S_Pointer	  s,
					  long		  prev_func,
					  long		  eval_type,
					  S_Pointer	  exec );

extern MDM_jump_t        MDM_object     ( CONST unsigned long index );


/*** Prototypen zum Debuggen (for wizards only) *******************************/

extern long		 MDMV_pmod_num;           /* Anzahl der Pseudomodule  */
extern MDM_pmod_t	 MDMV_pmod[];             /* Liste  der Pseudomodule  */
extern CONST long	 MDMV_object_tab_len;     /* Laenge der Adresstabelle */
extern CONST MDM_jump_t	 MDMV_object_tab[];       /* Adresstabelle des Kerns  */


#endif /* __MDM_base__ */
